package sociogram;
import java.util.List;
import java.util.LinkedList;
import simulation.*;
/**
 * The blackboard where the agents implicitly interact with 
 * the environment and between each others. In this case, 
 * the blacbloard contains the sociogram of students.
 * @author Iván García-Magariño
 */
public class Blackboard {
    /** Sociogram of the individual agents **/
    private Sociogram sociogram;
    /** GUI of the sociogram **/
    private SociogramGUI sociogramGUI=null;
    /** Simulation, which is used to get the number of the iteration of the 
     * simulation */
    private Simulation simulation;

    
    
    /** Single object of this class, using the Singleton 
     * pattern **/
    private static Blackboard instance;
    /** Constructor of the class. The constructor is
     private because of the Singleton pattern. **/
    private Blackboard(){
      
    }
    /** Provides the unique instance of this class according
     * to the Singleton pattern.
     */
    public static Blackboard getInstance(){
        if(instance==null){
            instance=new Blackboard();
        }
        return instance;
    }
    /** Returns the sociogram **/
    public Sociogram getSociogram(){
        return sociogram;
    }
    /** Sets the sociogram GUI */
    public void setSociogramGUI(SociogramGUI sociogramGUI) {
        this.sociogramGUI = sociogramGUI;
    }
    /** Returns the sociogram GUI**/
    public SociogramGUI getSociogramGUI(){
        return sociogramGUI;
    }
   
    /** Creates a sociogram from a list of individual agents **/
    public void createSociogram(List<IndividualAgent> list){
        sociogram=new Sociogram(list);
    }
    
    // GETTERS AND SETTERS
    public void setSimulation(Simulation simulation) {
        this.simulation = simulation;
    }

    public Simulation getSimulation() {
        return simulation;
    }
    
    
    
}
