/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sociogram;
import java.util.*;
import simulation.Agent;
import deck.*;

/**
 * Agent that represents a student unit with the different patterns.
 * @author Iván García-Magariño
 */
public class IndividualAgent implements Agent{
    /** Name of the student **/
    private String name;
     /** Type of individual agent **/
    private AgentType type;
    /** Identifier that is used in the Sociogram for its identification. **/
    private int index;
    /** Coordinates in the graphical position of the sociogram **/
    private double x,

    /**
     * Coordinates in the graphical position of the sociogram
     */
    y;
    
    /** The size limits of each size level. This size level is used in the
     * matrices suitable, unsuitable and change. For instance, {8} 
     * indicates there are two size levels 0 and 1, and their intervals of 
     * individuals are [0-8] and [9-Infinite].As another example {8, 16} 
     * indicates there are three size levels 0, 1 and 2, and their intervals of 
     * individuals are [0-8], [9-16] and [17, Infinite] The minimum limit of the
     * first size level is zero, and the maximum limit of the las size value is 
     * Infinite. Thus this vector only indicates the frontiers between size intervals
     * of the size levels.
     */
    private int sizeLimits[]={8};
    /** Constants of factor of suitability. It can be any number and multiplies the 
     * proability of change. It has different values for the differente size 
     * levels **/
    //public static double SUITABLE_FACTOR[]= {2, 1}; // {2.5, 0.4};
    /** Constants of factor of unsuitability. It can be any number and multiplies the 
     * proability of change. It has different values for the differente size 
     * levels **/
    //public static double UNSUITABLE_FACTOR[]= {2, 1}; //{2, 0.4};
    
    /** Constants for considering the difference between the short-terms 
     * terms and long-term relationships. The probabiliy of changing is 
     * CST_TREND/NumIterations
     * It has different values for the different size levels **/
    //public static double[] CST_TREND={10, 10};
    //public static double[] CST_TREND={1000, 1000};
    
    /** Constants for making the sociograms coherent with a high number of
     * reciprocal relations. Its values represent proportions that when
     * a relation changes the trend in one side, the other side also acquires
     * the same trend. The values are within the [0,1] interval.
     * It has different values for the different size levels **/
    //public static double CST_COHERENCE[]={0.50, 0.33};
    
    /** Size of decks for affinity (i.e. suitability and unsuitability). **/
    private static int sizeDeckAffinity=1000;
    /** Proportion of social selections with other agent of other type.
     The indexes are (a) the first type of agent, and the (b) second type of agent. **/
    private static int suitable[][]=
    /*                  QUIET   PARTICIPANT TANGENT     JOKER   OBSTRUCTIVE OCCASIONAL */
    /* QUIET */         {{60,    50,          50,          50,      50,          50},                                                                     
    /* PARTICIPANT */   {50,     60,          30,          50,      0,          50},
    /* TANGENT */       {50,     30,          60,          100,     50,          50},
    /* JOKER */         {50,     30,          10,         60,      50,          50},
    /* OBSTRUCTIVE */   {50,     0,          50,          50,      60,          50},
    /* OCCASIONAL */    {50,     50,          50,          50,      50,          60}};
  
     /** Proportion of social rejection with other agent of other type 
      * The indexes are (a) the first type of agent, and the (b) second type of agent. **/
    private static int unsuitable[][]=
    /*                  QUIET   PARTICIPANT TANGENT     JOKER   OBSTRUCTIVE OCCASIONAL */
    /* QUIET */         {{0,    1,          1,          1,      1,          1},                                                                     
    /* PARTICIPANT */   {1,     0,          2,          2,      3,          1},
    /* TANGENT */       {1,     2,          0,          1,      1,          1},
    /* JOKER */         {1,     2,          1,          0,      1,          1},
    /* OBSTRUCTIVE */   {1,     3,          1,          1,      0,          1},
    /* OCCASIONAL */    {1,     1,          1,          1,      1,          0}};
    
    /** Size of the deck for coherence for selections**/
    private static int sizeDeckCoherenceSelection= 100;
    /** Proportion for the coherence for selections**/
    private static int coherenceSelection =50;
    
    /** Size of the deck for coherence for rejections**/
    private static int sizeDeckCoherenceRejection= 100;
    /** Proportion for the coherence **/
    private static int coherenceRejection=1;
    
    
    
    //////// Atributes for Deck Random generators ////////
    /** The random generators for the coherence of selections**/
    private static DeckRandom deckCoherenceSelection = null;
    /** The random generators for the coherence of rejections**/
    private static DeckRandom deckCoherenceRejection = null;
    /** The random generators for the suitability and unsuitability between
     agent types. It has three choices for respectively (1) suitability (i.e.
     selection), (2) usuitability (i.e. rejection), and (3) neutral relations**/
    private static DeckRandom [][] deckAffinity = null;
    /** The selection choice from suitability **/
    private static final int SUITABLE_CHOICE= DeckRandom.FIRST_CHOICE;
    /** The rejection choice from suitability **/
    private static final int UNSUITABLE_CHOICE= DeckRandom.SECOND_CHOICE;
    /** The neutral choice **/
    private static final int NEUTRAL_CHOICE= DeckRandom.THIRD_CHOICE;
    

    
    
    
     /** Constructor of the class. Crates a tourist unit of the
     type indicated in the parameter. In the families and
     groups of friends, the number of people is randomly 
     selected within the limits established in the constants**/
    public IndividualAgent(AgentType type){
        this.type=type;
        x=-1;
        y=-1;
        // Creates the decks
        createDecks();
        
    }
    
    /** Obtains the number of size levels **/
    public int getNumSizeLevels(){
        return (sizeLimits.length+1);
    }
    
    
    /** Returns the size level, by checking in which limits is bounded the size
     * of the current simulation.
     */
   public int sizeLevel(){
       int sizeSimulation=Blackboard.getInstance().getSimulation().getSize();
       int level=0;
       while(level<sizeLimits.length && sizeLimits[level]<sizeSimulation){
           level++;
       }
       return level;
   }
    
    /** Indicates the probability that two types of agents are suitable for 
     selection, in a specific size level considering the group of agents of 
     the simulation.*/
/*    public double suitableTypes(int sizeLevel, AgentType typeA, AgentType typeB){
        double matrixValue=suitable[sizeLevel][intValue(typeA)][intValue(typeB)];
        return SUITABLE_FACTOR[sizeLevel] * matrixValue;
    }
 */
      /** Indicates the probability that two types of agents are suitable for
       rejection, in a specific size level considering the group of agents of 
     the simulation.*/
/*
   public double unsuitableTypes(int sizeLevel, AgentType typeA, AgentType typeB){
        double matrixValue=unsuitable[sizeLevel][intValue(typeA)][intValue(typeB)];
        return UNSUITABLE_FACTOR[sizeLevel] * matrixValue;
    }
*/    
    /** Convert a type of an agent into an integer for accessing the 
     suitability of two types of agents**/
    public static int intValue(AgentType type){
        switch(type){
            case QUIET: return 0;
            case PARTICIPANT: return 1;
            case TANGENT: return 2;
            case JOKER: return 3;
            case OBSTRUCTIVE: return 4;
            case OCCASIONAL: return 5;
            default:return -1;
        }
    }
    
   
    /** This method is periodically called withing the 
     * multi-agent system simulation. In this case,
     * the agent makes the social connections according
     * to several matrices and constants that define its 
     * behavior regarding the types of agents and other 
     * factors.
     */
    public void live(){
        // Variables
        double rand, trend; // selected, rejected,
        int level;
        // Get the size level regarding the size of the simulation
        level = sizeLevel();
        // Get the agents 
        IndividualAgent agents[]=Blackboard.getInstance().getSociogram().getAgents();
        IndividualAgent agent;
        Random r = new Random();
        Sociogram sociogram=Blackboard.getInstance().getSociogram();
        // Only the ones with an index below, to reduce variability because
        // of simulating both directions unnecessarily
        for(int i=0; i<this.getIndex();i++){
            // Get Each agent
            agent=agents[i];
            // If different agent evaluates the posibility
            if (this.getIndex()!=agent.getIndex()){
                // It considers the probability of change the trend
                //rand= r.nextDouble();
                //if(rand<probabilityOfChange(level, this.getType(),agent.getType())){
                /*
                if(deckChange[level][intValue(this.getType())]
                        [intValue(agent.getType())].nextInt()==DeckRandom.YES){
                    // It considers the difference beteween short-term and
                    // long term relationships
                    rand= r.nextDouble();
                    int iteration=Blackboard.getInstance().getSimulation().getIteration();
                    if(rand<(CST_TREND[level]/iteration)){
                        // Update the trend according to the suitability and unsuitabilaty
                        /*rand= r.nextDouble();
                        selected=suitableTypes(level, this.getType(),agent.getType());
                        rejected=unsuitableTypes(level, this.getType(),agent.getType());
                        if(rand<selected){
                            trend=1.0;                            
                        }else if (rand>(1-rejected)){
                            trend=-1.0;
                        }else{
                            trend=0;
                        }*
                */
                        int choice = deckAffinity[intValue(this.getType())]
                                [intValue(agent.getType())].nextInt();
                        if(choice==SUITABLE_CHOICE){
                            trend=1.0;
                        }else if(choice==UNSUITABLE_CHOICE){
                            trend=-1.0;
                        }else{
                            trend=0;
                        }
                        sociogram.setTrend(this, agent,trend);
                       
                       // If the trend is not zero, update the reciprocal trend 
                        // considering the CST_COHERENCE constant.
                        if(trend>0.5){
                            if(deckCoherenceSelection.nextInt()==DeckRandom.YES){
                                sociogram.setTrend(agent, this, trend);
                                sociogram.updateConnection(agent, this, trend);
                            }
                        }else if(trend<(-0.5)){
                            if(deckCoherenceRejection.nextInt()==DeckRandom.YES){
                                sociogram.setTrend(agent, this, trend);
                                sociogram.updateConnection(agent, this, trend);
                            }
                        }
                    //}
                // Update the relationship based on the trend
                sociogram.updateConnection(this, agent, sociogram.getTrend(this, agent));
            } // End if   
        }
        // Refress the Sociogram GUI for the animation
        if(Blackboard.getInstance().getSociogramGUI()!=null){
            Blackboard.getInstance().getSociogramGUI().repaint();
        }
    }
    /**
     * It creates all the Deck Random generators.
     */
    private static void createDecks(){
        // Only create the decks the first time
        if(deckAffinity==null){
            // Creates a factory of deck random generators
            DeckFactory factory= new DeckFactory();
            // Gets sizes
            //int numLevels=getNumSizeLevels();
            int numTypes= AgentType.values().length;        
            // Create the generators for coherence
            deckCoherenceSelection= new DeckRandom(coherenceSelection,sizeDeckCoherenceSelection);
            deckCoherenceRejection= new DeckRandom(coherenceRejection,sizeDeckCoherenceRejection);
            /*
            deckCoherence = new DeckRandom[numLevels];
            for (int i=0; i< numLevels;i++){
                deckCoherence[i]=factory.createDeck(CST_COHERENCE[i], size);
                System.out.print("Prob "+CST_COHERENCE[i]+". ");
                        System.out.println("deckCoherence ["+i+"]="
                                +deckCoherence[i]);
            }
            // Creates the generators for change
            deckChange = new DeckRandom[numLevels][numTypes][numTypes];
            for (int i=0; i<numLevels; i++){
                for (int j=0; j<numTypes; j++){
                    for (int k=0; k<numTypes; k++){
                        double value = change[i][j][k]*CHANGE_FACTOR[i];
                        deckChange[i][j][k]=factory.createDeck(value, size);
                        System.out.print("Prob "+value+". ");
                        System.out.println("deckChange ["+i+"]["+j+"]["+k+"]="
                                +deckChange[i][j][k]);
                    }
                }
            }
            */
            // Creates the generators for suitable/unsuitable
            deckAffinity = new DeckRandom[numTypes][numTypes];
            for (int i=0; i<numTypes; i++){
                for (int j=0; j<numTypes; j++){
                        int propSuitable = suitable[i][j];
                        int propUnsuitable = unsuitable[i][j];
                        deckAffinity[i][j]= new DeckRandom(propSuitable, 
                                propUnsuitable, sizeDeckAffinity);
                        //System.out.print("Probs "+probSuitable+","+probUnsuitable+". ");
                        //System.out.println("deckSuitable ["+i+"]["+j+"]["+k+"]="
                        //        +deckAffinity[i][j][k]);                    
                }
            }
        } // End if deck is null
    }
    
    /** It sets the value of a deck for a specific choice.
     * @param nameDeckChoice A name that identifies a deck and
     * specific choices. This parameter can be "suitable",
     * "unsuitable", "coherenceSelection" or "coherenceRejection".
     * @param indexes If the deck choice values are stored
     * in a vector/matrix of one or several directions, this 
     * parameter represents the position in this vector/matrix. 
     * Otherwise, this parameter is ignored. It also sets the simetric
     * position, which makes sense with the simiplifications in which
     * only the i, j is consulted when i>j.
     * @param value Value that is assigned to the specific
     * choice of the particular deck.
     */
    public static void setDeckValue(String nameDeckChoice, int[] indexes, int value) {
        // Create the decks if these are not created
        if(deckAffinity==null){
            createDecks();
        }
        // Change the corresponding value, and create againg the corresponding
        // deck
        int i, j;
        if(nameDeckChoice.equals("suitable") || nameDeckChoice.equals("unsuitable")){
            i=indexes[0];
            j=indexes[1];
            if(nameDeckChoice.equals("suitable")){
                suitable[i][j]=value;
                suitable[j][i]=value;
            }else{
                unsuitable[i][j]=value;
                unsuitable[j][i]=value;
            }
            int propSuitable = suitable[i][j];
            int propUnsuitable = unsuitable[i][j];
            deckAffinity[i][j]= new DeckRandom(propSuitable, 
                              propUnsuitable, sizeDeckAffinity);
            deckAffinity[j][i]=deckAffinity[i][j];
        }else if(nameDeckChoice.equals("coherenceSelection")){
            coherenceSelection=value;
            deckCoherenceSelection= new DeckRandom(coherenceSelection,sizeDeckCoherenceSelection);            
        }else if(nameDeckChoice.equals("coherenceRejection")){
            coherenceRejection=value;
            deckCoherenceRejection= new DeckRandom(coherenceRejection,sizeDeckCoherenceRejection);
        }else{
            System.out.println("setDeckValue: wrong parameter:"+nameDeckChoice);
        }
    }
    
    /** It gets the value of a deck for a specific choice.
     * @param nameDeckChoice A name that identifies a deck and
     * specific choices. This parameter can be "suitable",
     * "unsuitable", "coherenceSelection" or "coherenceRejection".
     * @param indexes If the deck choice values are stored
     * in a vector/matrix of one or several directions, this 
     * parameter represents the position in this vector/matrix. 
     * Otherwise, this parameter is ignored.
     * @return Value of the specific choice of the particular deck.
     * It returns -1 in case of any error.
     */
    public static int getDeckValue(String nameDeckChoice, int[] indexes) {
        // Check if the the decks have been created
        if(deckAffinity==null){
            return -1;
        }
        // Change the corresponding value, and create againg the corresponding
        // deck
        int i, j;
        int value=-1;
        if(nameDeckChoice.equals("suitable") || nameDeckChoice.equals("unsuitable")){
            i=indexes[0];
            j=indexes[1];
            if(nameDeckChoice.equals("suitable")){
                value=suitable[i][j];
            }else{
                value=unsuitable[i][j];
            }
            int propSuitable = suitable[i][j];
            int propUnsuitable = unsuitable[i][j];
            deckAffinity[i][j]= new DeckRandom(propSuitable, 
                              propUnsuitable, sizeDeckAffinity);
        }else if(nameDeckChoice.equals("coherenceSelection")){
            value=coherenceSelection;
        }else if(nameDeckChoice.equals("coherenceRejection")){
            value=coherenceRejection;
        }
        return value;
    }
    
    /// Getters and Setters
    public String getName() {
        return name;
    }

    public int getIndex() {
        return index;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }
    public void setX(double x) {
        this.x = x;
    }

    public void setY(double y) {
        this.y = y;
    }
    public AgentType getType() {
        return type;
    }
    
    
    public static int getSizeDeckAffinity() {
        return sizeDeckAffinity;
    }

    public static void setSizeDeckAffinity(int sizeDeckAffinity) {
        IndividualAgent.sizeDeckAffinity = sizeDeckAffinity;
    }

    public static int getSizeDeckCoherenceSelection() {
        return sizeDeckCoherenceSelection;
    }

    public static void setSizeDeckCoherenceSelection(int sizeDeckCoherenceSelection) {
        IndividualAgent.sizeDeckCoherenceSelection = sizeDeckCoherenceSelection;
    }

    public static int getSizeDeckCoherenceRejection() {
        return sizeDeckCoherenceRejection;
    }

    public static void setSizeDeckCoherenceRejection(int sizeDeckCoherenceRejection) {
        IndividualAgent.sizeDeckCoherenceRejection = sizeDeckCoherenceRejection;
    }
    
    /// End Getters and Setters

    
}
