/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package trainingsoci;
import java.util.*;
import trainingdeck.*;
import simulation.*;
import sociogram.*;

/**
 * It represents the ABS-SOCI simulator implementing the 
 * necessary interface for being trained.
 * @author Iván García-Magariño
 */
public class TrainableSoci implements TrainableSim{
    /** Simulation of ABS-SOCI **/
    private Simulation simulation;
    /** The number of iterations is fixed to one, because
     * the simulation has been simplied according to
     * ATABS approach with Deck random generators
     */
    private static final int NUM_ITERATIONS=1;
    
    /** It runs a simulation with the input and output in a
     * generic format for being trained.
     * @param input The input parameters of simulation with 
     * a Map with their names and their values. These parameters 
     * are "quite", "participant", "tangent", "joker", "obstructive"
     * and "occasional".
     * @return The result (i.e. output) of the simulation with
     * a Map with the names of their features (e.g. metrics)
     * and their values.
     */
    @Override
    public HashMap<String, Object> runSimulation(
            HashMap<String, Object> input) {
        // Creates a simulation
        simulation = new Simulation();
        Blackboard.getInstance().setSimulation(simulation);
        // Creates the agents and register these in the simulation
        createAgents(input);
        // Run the simulation
        simulation.run(NUM_ITERATIONS);
        // It obtains the results and provides these.
        Sociogram sociogram = Blackboard.getInstance().getSociogram();
        HashMap<String, Object> output = new HashMap<String, Object>();
        output.put("IAg", new Double(sociogram.measureIAg()));
        output.put("ICg", new Double(sociogram.measureICg()));
        output.put("IDg", new Double(sociogram.measureIDg()));
        output.put("IIg", new Double(sociogram.measureIIg()));
        return output;
    }
    /** It creates a list of agent for a certain simulation input 
     * @param input Simulation Input
     */
    private void createAgents( HashMap<String, Object> input){
        // Create an empty list of agents
        List<IndividualAgent> list=new LinkedList<IndividualAgent>();
        IndividualAgent agent;
        // Creates agents of each type and add these to the sociogram and simulation
        int numAgents;
        numAgents=((Integer)input.get("quiet")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.QUIET);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=((Integer)input.get("participant")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.PARTICIPANT);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=((Integer)input.get("tangent")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.TANGENT);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=((Integer)input.get("joker")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.JOKER);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=((Integer)input.get("obstructive")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.OBSTRUCTIVE);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=((Integer)input.get("occasional")).intValue();
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.OCCASIONAL);
            simulation.register(agent);
            list.add(agent);
        }
        // Creates a sociogram from the list of agents
        Blackboard.getInstance().createSociogram(list);
        
    }
    /** It sets the value of of a deck for a specific choice.
     * @param nameDeckChoice A name that identifies a deck and
     * specific choices.
     * @param indexes If the deck choice values are stored
     * in a vector/matrix of one or several directions, this 
     * parameter represents the position in this vector/matrix. 
     * Otherwise, this parameter is ignored.
     * @param value Value that is assigned to the specific
     * choic of the particular deck.
     */
    @Override
    public void setDeckValue(String nameDeckChoice, int[] indexes, int value) {
        IndividualAgent.setDeckValue(nameDeckChoice, indexes, value);
    }
    
    /** It gets the value of a deck for a specific choice.
     * @param nameDeckChoice A name that identifies a deck and
     * specific choices. This parameter can be "suitable",
     * "unsuitable", "coherenceSelection" or "coherenceRejection".
     * @param indexes If the deck choice values are stored
     * in a vector/matrix of one or several directions, this 
     * parameter represents the position in this vector/matrix. 
     * Otherwise, this parameter is ignored.
     * @return Value of the specific choice of the particular deck.
     * It returns -1 in case of any error.
     */
    @Override
    public int getDeckValue(String nameDeckChoice, int[] indexes) {
        return IndividualAgent.getDeckValue(nameDeckChoice, indexes);
    }
    
    
}
